// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart' as bloc_base;
import 'package:wyatt_crud_bloc/wyatt_crud_bloc.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

mixin CrudMixin<Bloc extends bloc_base.StateStreamable<CrudState>,
    SuccessState extends CrudSuccess> {
  String? get loaderComponentId => null;
  String? get errorComponentId => null;

  Widget errorBuilder(BuildContext context, CrudError state) =>
      context.components.errorComponent(errorComponentId).call(
            message:
                (state.message != null) ? TextWrapper(state.message!) : null,
          );

  Widget loadingBuilder(BuildContext context, CrudLoading state) =>
      context.components.loaderComponent(loaderComponentId).call();

  Widget initialBuilder(BuildContext context, CrudInitial state) =>
      const SizedBox.shrink();

  Widget successBuilder(BuildContext context, SuccessState state);

  Widget crudBuilder(BuildContext context, CrudState state) =>
      CrudBuilder<Bloc, CrudInitial, CrudLoading, SuccessState, CrudError>(
        errorBuilder: errorBuilder,
        loadingBuilder: loadingBuilder,
        initialBuilder: initialBuilder,
        builder: successBuilder,
      );
}
