// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_continuous_deployment/src/core/commands/build_app/commands/build_ipa/build_ipa_task.dart';
import 'package:wyatt_continuous_deployment/src/core/commands/deploy_app/commands/deploy_ios/deploy_ios_task.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';

class BuildAndDeployIosAppTask extends TaskCubit {
  factory BuildAndDeployIosAppTask() => BuildAndDeployIosAppTask._(
        checkToolsUsecase: getIt(),
        checkConfigFileUsecase: getIt(),
      );

  BuildAndDeployIosAppTask._({
    required super.checkToolsUsecase,
    required super.checkConfigFileUsecase,
  });

  @override
  Future<void> onRun(Context ctx) async {
    // 1. Build ipa
    final buildIpaTask = BuildIpaTask();
    await buildIpaTask.onRun(ctx);
    if (buildIpaTask.status != 0) {
      return emit(buildIpaTask.state);
    }

    // 2. Deploy it
    final deployIosTask = DeployIosTask();
    await deployIosTask.onRun(ctx);
    if (deployIosTask.status != 0) {
      return emit(deployIosTask.state);
    }
  }
}
