// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:meta/meta.dart';
import 'package:wyatt_continuous_deployment/src/core/constants/app_constants.dart';
import 'package:wyatt_continuous_deployment/src/core/enums/flag.dart';
import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';

abstract class TaskCommand extends Command<int> {
  TaskCommand({
    required this.task,
  });

  final TaskCubit task;

  /// [ArgResults] used for testing purposes only.
  @visibleForTesting
  ArgResults? testArgResults;

  /// [ArgResults] for the current command.
  ArgResults get results => testArgResults ?? argResults!;

  /// Context object for the current command.
  Context get context => Context(
        currentDirectory: Directory.current,
        configFile: () {
          // Check if config flag is set.
          final wyattPath = globalResults![AppConstants.configArg] as String?;

          final wyattFile = File(wyattPath ?? AppConstants.configFileName);

          return wyattFile;
        }.call(),
        flags: Flag.fromArgs(results: results, globalResults: globalResults!),
        environment: () {
          final env = <String, String>{}..addAll(Platform.environment);

          // Load environment variables from the dot file.
          final dotFile = File('.env');
          if (dotFile.existsSync()) {
            final dotFileContent = dotFile.readAsStringSync();
            final lines = dotFileContent.split('\n');
            for (final line in lines) {
              final parts = line.split('=');
              if (parts.length == 2) {
                env[parts[0].trim()] = parts[1].trim();
              }
            }
          }

          return env;
        }.call(),
      );

  @override
  FutureOr<int>? run() async {
    await task.run(context);
    await task.awaitForFinished();
    return task.status;
  }
}
