// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:io';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/domain/usecases/usecase.dart';

@usecase
class CheckConfigFileUsecase extends AsyncUseCase<String, void> {
  const CheckConfigFileUsecase() : super();

  @override
  FutureOr<void> onStart(String? params) {
    if (params == null) {
      throw const ClientException(
        'CheckConfigFileUsecase: You must provide path',
      );
    }
  }

  @override
  FutureOrResult<void> execute(String? params) => unsafeAsync(() {
        final configFile = File(params!);
        if (!configFile.existsSync()) {
          throw ClientException(
            'CheckConfigFileUsecase: Cannot find $params',
          );
        }
      });
}
