// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:wyatt_i18n/wyatt_i18n.dart';

/// Provides extensions for [BuildContext].
extension BuildContextExtension on BuildContext {
  /// Returns the current locale of the app.
  String get locale => Localizations.localeOf(this).languageCode;

  /// Returns the current [I18n] instance.
  ///
  /// Throws a [NotLoadedException] if the [I18n] instance is not loaded.
  I18n get i18n {
    final i18n = Localizations.of<I18n>(this, I18n);
    if (i18n == null) {
      throw NotLoadedException();
    }

    return i18n;
  }
}
