// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template file_selection_button_theme_extension}
/// Default file selection button theme extension using Flutter's
/// default theme and opinionated defaults.
/// {@endtemplate}
class FileSelectionButtonThemeExtensionDefault
    extends FileSelectionButtonThemeExtension {
  /// {@macro file_selection_button_theme_extension}
  const FileSelectionButtonThemeExtensionDefault({
    required super.disabledStyle,
    required super.focusedStyle,
    required super.hoveredStyle,
    required super.normalStyle,
    required super.tappedStyle,
    required super.selectedStyle,
    required super.invalidStyle,
  });

  /// Creates a [FileSelectionButtonThemeExtensionDefault] given a [ThemeData]
  ///
  /// {@macro file_selection_button_theme_extension}
  factory FileSelectionButtonThemeExtensionDefault.from(ThemeData theme) {
    final style = FileSelectionButtonStyle(
      titleStyle: theme.textTheme.labelLarge,
      subtitleStyle: theme.textTheme.labelSmall,
      radius: (theme.buttonTheme.shape is RoundedRectangleBorder)
          ? (theme.buttonTheme.shape as RoundedRectangleBorder).borderRadius
          : null,
      padding: const EdgeInsets.symmetric(horizontal: 10),
      foregroundColors: MultiColor.single(theme.colorScheme.onPrimary),
      backgroundColors: MultiColor.single(theme.colorScheme.primary),
      animationDuration: Duration.zero,
    );

    return FileSelectionButtonThemeExtensionDefault(
      normalStyle: style,
      focusedStyle: style,
      hoveredStyle: style.copyWith(
        backgroundColors:
            MultiColor.single(theme.colorScheme.primary.withOpacity(0.80)),
      ),
      tappedStyle: style.copyWith(
        backgroundColors:
            MultiColor.single(theme.colorScheme.primary.withOpacity(0.75)),
      ),
      selectedStyle: style,
      disabledStyle: style.copyWith(
        foregroundColors:
            MultiColor.single(theme.colorScheme.onSurface.withOpacity(0.12)),
        backgroundColors:
            MultiColor.single(theme.colorScheme.onSurface.withOpacity(0.38)),
      ),
      invalidStyle: style,
    );
  }

  /// Creates a [FileSelectionButtonThemeExtensionDefault] from a dark theme
  factory FileSelectionButtonThemeExtensionDefault.dark() =>
      FileSelectionButtonThemeExtensionDefault.from(ThemeData.dark());

  /// Creates a [FileSelectionButtonThemeExtensionDefault] from a light theme
  factory FileSelectionButtonThemeExtensionDefault.light() =>
      FileSelectionButtonThemeExtensionDefault.from(ThemeData.light());
}
