// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_layout/src/presentation/layouts/layout.dart';
import 'package:wyatt_ui_layout/wyatt_ui_layout.dart';

typedef ComponentCallBack<T extends Component> = T? Function(T? component);

/// {@template structural_layout}
/// An abstract class that provides a base for creating custom structural layout
/// widgets.
/// {@endtemplate}
abstract class StructuralLayout extends Layout {
  /// {@macro structural_layout}
  const StructuralLayout({super.key});

  factory StructuralLayout.withTopAppBar({
    required Widget body,
    ComponentCallBack<TopAppBarComponent>? custom,
    String? barId,
    double height = 60,
    ScaffoldFieldsWrapper? scaffoldFieldsWrapper,
  }) =>
      TopAppBarLayout(
        body: body,
        barId: barId,
        custom: custom,
        height: height,
        scaffoldFieldsWrapper: scaffoldFieldsWrapper,
      );

  factory StructuralLayout.withTopNavigationBar({
    required Widget body,
    ComponentCallBack<TopNavigationBarComponent>? custom,
    String? barId,
    double height = 60,
    ScaffoldFieldsWrapper? scaffoldFieldsWrapper,
  }) =>
      TopNavigationBarLayout(
        body: body,
        barId: barId,
        custom: custom,
        height: height,
        scaffoldFieldsWrapper: scaffoldFieldsWrapper,
      );

  factory StructuralLayout.withFrame({
    required Widget body,
    ComponentCallBack<TopAppBarComponent>? customAppBar,
    ComponentCallBack<BottomNavigationBarComponent>? customBottomNavBar,
    ComponentCallBack<FloatingActionButtonComponent>?
        customFloatingActionButton,
    String? appBarId,
    String? bottomNavBarId,
    String? floatingActionButtonId,
    double height = 60,
    ScaffoldFieldsWrapper? scaffoldFieldsWrapper,
  }) =>
      FrameLayout(
        body: body,
        customAppBar: customAppBar,
        customBottomNavBar: customBottomNavBar,
        customFloatingActionButton: customFloatingActionButton,
        appBarId: appBarId,
        bottomNavBarId: bottomNavBarId,
        floatingActionButtonId: floatingActionButtonId,
        height: height,
        scaffoldFieldsWrapper: scaffoldFieldsWrapper,
      );

  factory StructuralLayout.withBottomNavBar({
    required Widget? body,
    BottomNavigationBarComponent? Function(BottomNavigationBarComponent?)?
        custom,
    String? barId,
    ScaffoldFieldsWrapper? scaffoldFieldsWrapper,
  }) =>
      BottomNavigationBarLayout(
        body: body,
        custom: custom,
        barId: barId,
        scaffoldFieldsWrapper: scaffoldFieldsWrapper,
      );
}
